/*
PC7166 diagnostic software
This program makes sure the counters can be accessed on the PC7166 board.
*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>
#include <math.h>


typedef unsigned char BYTE;
typedef unsigned int WORD;

#define DEF_BASE	0X300	/* default base address of interface board */

#define DATA1			0			/* data register of LS7166 */
#define CONTROL1	1			/* control register of LS7166 */
#define DATA2			2			/* data register of LS7166 */
#define CONTROL2	3			/* control register of LS7166 */
#define DATA3			4			/* data register of LS7166 */
#define CONTROL3	5			/* control register of LS7166 */
#define DATA4			6			/* data register of LS7166 */
#define CONTROL4	7			/* control register of LS7166 */

#define LATCH			8			/* a read causes all LS7166s to latch counter */

#define MASTER_RESET	0X20	/* master reset command */
#define INPUT_SETUP		0X68	/* command to setup counter input */
#define QUAD_X1				0XC1	/* command to setup quadrature multiplier to 1 */
#define QUAD_X2				0XC2	/* command to setup quadrature multiplier to 2 */
#define QUAD_X4				0XC3	/* command to setup quadrature multiplier to 4 */
#define ADDR_RESET		0X01	/* command to reset address pointer */
#define LATCH_CNTR		0X02	/* command to latch counter */
#define CNTR_RESET		0X04	/* command to reset counter */
#define PRESET_CTR    0X08	/* transfer preset to counter */

/* Function prototypes	*/

int  test(void);						/* test if counters can be accessed */

/* Global variables */

int  base = DEF_BASE;		/* base address of interface board */

/* Main program */
int main(void)
{
	clrscr();

	printf("US Digital Corp.  PC7166 diagnostics");

	gotoxy(1,3); printf("Enter address of PC7166 card (hex): ");
	scanf("%x", &base);

	if (test() == 0) {
		printf("\n\nTest passed: the four counters can be accessed.");
	}

	return 0;
}



int test(void) {	/* test if the counters can be accessed */

	int fail1 = 0, fail2 = 0, fail3 = 0, fail4 = 0;

	gotoxy(1,5);
	/* preset a value and read it back */
	outportb(base + CONTROL1, ADDR_RESET);
	outportb(base + DATA1, 0X11);
	outportb(base + DATA1, 0X55);
	outportb(base + DATA1, 0XAA);
	outportb(base + CONTROL1, PRESET_CTR);
	outportb(base + CONTROL1, LATCH_CNTR);
	outportb(base + CONTROL1, ADDR_RESET);
	if (inportb(base + DATA1) != 0X11) fail1++;
	if (inportb(base + DATA1) != 0X55) fail1++;
	if (inportb(base + DATA1) != 0XAA) fail1++;
	if (fail1) printf("\nCounter 1 fails at address %X", base + DATA1);
	/* preset a value and read it back */
	outportb(base + CONTROL2, ADDR_RESET);
	outportb(base + DATA2, 0X22);
	outportb(base + DATA2, 0X55);
	outportb(base + DATA2, 0XAA);
	outportb(base + CONTROL2, PRESET_CTR);
	outportb(base + CONTROL2, LATCH_CNTR);
	outportb(base + CONTROL2, ADDR_RESET);
	if (inportb(base + DATA2) != 0X22) fail2++;
	if (inportb(base + DATA2) != 0X55) fail2++;
	if (inportb(base + DATA2) != 0XAA) fail2++;
	if (fail2) printf("\nCounter 2 fails at address %X", base + DATA2);
	/* preset a value and read it back */
	outportb(base + CONTROL3, ADDR_RESET);
	outportb(base + DATA3, 0X33);
	outportb(base + DATA3, 0X55);
	outportb(base + DATA3, 0XAA);
	outportb(base + CONTROL3, PRESET_CTR);
	outportb(base + CONTROL3, LATCH_CNTR);
	outportb(base + CONTROL3, ADDR_RESET);
	if (inportb(base + DATA3) != 0X33) fail3++;
	if (inportb(base + DATA3) != 0X55) fail3++;
	if (inportb(base + DATA3) != 0XAA) fail3++;
	if (fail3) printf("\nCounter 3 fails at address %X", base + DATA3);
	/* preset a value and read it back */
	outportb(base + CONTROL4, ADDR_RESET);
	outportb(base + DATA4, 0X44);
	outportb(base + DATA4, 0X55);
	outportb(base + DATA4, 0XAA);
	outportb(base + CONTROL4, PRESET_CTR);
	outportb(base + CONTROL4, LATCH_CNTR);
	outportb(base + CONTROL4, ADDR_RESET);
	if (inportb(base + DATA4) != 0X44) fail4++;
	if (inportb(base + DATA4) != 0X55) fail4++;
	if (inportb(base + DATA4) != 0XAA) fail4++;
	if (fail4) printf("\nCounter 4 fails at address %X", base + DATA4);

	if (fail1 && fail2 && fail3 && fail4) {
		printf("\n\nAll counters failed, check the address on the DIP switch");
		printf("\nand make sure that address does not interfere with any other");
		printf("\ncards in your system.");
	}
	else if (fail1 || fail2 || fail3 || fail4) {
		printf("\n\nSome counters failed, check the address on the DIP switch");
		printf("\nand make sure that address does not interfere with any other");
		printf("\ncards in your system.");
	}

	return fail1 + fail2 + fail3 + fail4;
}
