/* Simple demo: displays position of encoders connected to PC7166 board */
/* U.S. Digital Corp (206) 260-7451   12-2-93 */
/* Borland C source code */

#include <stdio.h>             
#include <dos.h>
#include <conio.h>

/* addresses */
#define DEF_BASE	0X300	/* default base address of interface board */

#define DATA1			0			/* data register of LS7166 */
#define CONTROL1	1			/* control register of LS7166 */
#define DATA2			2			/* data register of LS7166 */
#define CONTROL2	3			/* control register of LS7166 */
#define DATA3			4			/* data register of LS7166 */
#define CONTROL3	5			/* control register of LS7166 */
#define DATA4			6			/* data register of LS7166 */
#define CONTROL4	7			/* control register of LS7166 */

#define LATCH			8			/* a read causes all LS7166s to latch counter */

/* LS7166 commands */
#define MASTER_RESET	0X20	/* master reset command */
#define INPUT_SETUP		0X68	/* command to setup counter input */
#define QUAD_X1				0XC1	/* command to setup quadrature multiplier to 1 */
#define QUAD_X2				0XC2	/* command to setup quadrature multiplier to 2 */
#define QUAD_X4				0XC3	/* command to setup quadrature multiplier to 4 */
#define ADDR_RESET		0X01	/* command to reset address pointer */
#define LATCH_CNTR		0X02	/* command to latch counter */
#define CNTR_RESET		0X04	/* command to reset counter */
#define PRESET_CTR    0X08	/* transfer preset to counter */

/* Global variables */

int  base = DEF_BASE;		/* base address of interface board */


/* Functions */

void init(void) {       			/* initialize the LS7166s */
	outportb(base + CONTROL1, MASTER_RESET);
	outportb(base + CONTROL1, INPUT_SETUP);
	outportb(base + CONTROL1, QUAD_X4);
	outportb(base + CONTROL1, CNTR_RESET);

	outportb(base + CONTROL2, MASTER_RESET);
	outportb(base + CONTROL2, INPUT_SETUP);
	outportb(base + CONTROL2, QUAD_X4);
	outportb(base + CONTROL2, CNTR_RESET);

	outportb(base + CONTROL3, MASTER_RESET);
	outportb(base + CONTROL3, INPUT_SETUP);
	outportb(base + CONTROL3, QUAD_X4);
	outportb(base + CONTROL3, CNTR_RESET);

	outportb(base + CONTROL4, MASTER_RESET);
	outportb(base + CONTROL4, INPUT_SETUP);
	outportb(base + CONTROL4, QUAD_X4);
	outportb(base + CONTROL4, CNTR_RESET);
}


void hard_latch(void) {  					/* latch position with low pulse on pin 3 */
	outportb(base + LATCH, 0);       /* toggle pin 3 of LS 7166 */
}


long read_position1(void) {  			/* read position of encoder */
	long position;
	outportb(base + CONTROL1, ADDR_RESET);				/* reset address pointer */
	position  = (long)inportb(base + DATA1);      /* least significant byte */
	position += (long)inportb(base + DATA1) << 8;
	position += (long)inportb(base + DATA1) <<16; /* most significant byte */
	return position;
}


long read_position2(void) {  			/* read position of encoder */
	long position;
	outportb(base + CONTROL1, ADDR_RESET);				/* reset address pointer */
	position  = (long)inportb(base + DATA2);      /* least significant byte */
	position += (long)inportb(base + DATA2) << 8;
	position += (long)inportb(base + DATA2) <<16; /* most significant byte */
	return position;
}


long read_position3(void) {  			/* read position of encoder */
	long position;
	outportb(base + CONTROL1, ADDR_RESET);				/* reset address pointer */
	position  = (long)inportb(base + DATA3);      /* least significant byte */
	position += (long)inportb(base + DATA3) << 8;
	position += (long)inportb(base + DATA3) <<16; /* most significant byte */
	return position;
}


long read_position4(void) {  			/* read position of encoder */
	long position;
	outportb(base + CONTROL1, ADDR_RESET);				/* reset address pointer */
	position  = (long)inportb(base + DATA4);      /* least significant byte */
	position += (long)inportb(base + DATA4) << 8;
	position += (long)inportb(base + DATA4) <<16; /* most significant byte */
	return position;
}


void main(void) {

	clrscr();
	gotoxy(1,3); printf("Enter address of PC7166 card (hex): ");
	scanf("%x", &base);
	init();
	while (!kbhit()) {
		hard_latch();
		printf("\npos1 =%8lX", read_position1());
		printf("       pos2 =%8lX", read_position2());
		printf("       pos3 =%8lX", read_position3());
		printf("       pos4 =%8lX", read_position4());
	}
}
